## no critic( RequireBlockMap ProhibitMagicNumbers)
use strict;
use warnings;
use DE_EPAGES::Billsafe::API::Order;
use DE_EPAGES::Billsafe::API::Article;
use DE_EPAGES::Core::API::Error qw ( GetError ExistsError Error );

use DE_EPAGES::Database::API::Connection qw ( RunOnStore GetTestSite  );
use DE_EPAGES::Object::API::Factory qw (LoadObjectByPath ExistsObjectByPath LoadRootObject);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);
use DE_EPAGES::Billsafe::UI::BasketOrder;
use DE_EPAGES::Object::API::Language qw (GetPKeyLanguageByCode);
use DE_EPAGES::Core::API::Script qw (RunScript);

BEGIN { *Bubo:: = *DE_EPAGES::Billsafe::UI::BasketOrder:: }

use Test::More;

my @expected_order_keys = qw ( order_amount order_currencyCode order_taxAmount product );
my @expected_order_keys_but_undef = qw/ order_number /;

my @expected_art_keys = qw ( name number type quantity tax );
my @expected_art_keys_netto  = sort 'netPrice',   @expected_art_keys;
my @expected_art_keys_brutto = sort 'grossPrice', @expected_art_keys;
# optional: description quantityShipped
my $ROOTPATH = '/Test';

sub check_nvp {
    my( $ExpectedTaxModel, $order, %nvp,  ) = @_;
    # Build list of expected keys.
    my $num_articles = scalar $order->get_articles;
    my @expected_keys = (@expected_order_keys, @expected_order_keys_but_undef);

    if($ExpectedTaxModel eq 'gross') {
        for my $ak ( @expected_art_keys_brutto ) {
            push @expected_keys,
                map 'articleList_' . $_ . '_' . $ak, 0 .. $num_articles - 1;
        }
    }
    else {
        for my $ak ( @expected_art_keys_netto ) {
            push @expected_keys,
                map 'articleList_' . $_ . '_' . $ak, 0 .. $num_articles - 1;
        }
    }

    # Got all expexted keys? With defined values?
    for ( @expected_keys ) {
        ok exists $nvp{$_}, "$_ exists";
        ok defined $nvp{$_}, "$_ defined"
            unless $_ ~~ \@expected_order_keys_but_undef;
    }
    # Any unexpected keys?
    for ( keys %nvp ) {
        ok $_ ~~ \@expected_keys, "in list of expected keys: $_"
            unless $_ =~ m/(description|netPrice)$/;
    }
}


sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    return 1 if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    my $System = LoadRootObject();
    $System->insertChild( 'Object', { Alias => 'Test' } );
    ImportObjects('import.xml', $ROOTPATH);
    local $DE_EPAGES::Object::API::Language::LANGUAGEID = GetPKeyLanguageByCode('en');

    return 1;
}

sub run_tests {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
            # initializsation
            ok( Init(), 'initialization' );
            eval {
                _testImport();
            };
            if (ExistsError()) {
                my $Error = GetError();
                fail('Error - '. $Error->code);
                diag($Error->longMessage);
                # always cleanup
                ok( Cleanup(), 'cleanup');
                Error($Error);
            }
            # cleanup
            ok( Cleanup(), 'cleanup');
        }
    );
}


sub _testImport{
    my $Shop = LoadObjectByPath($ROOTPATH)->child('TestShopBillSafe');
    isa_ok($Shop, 'DE_EPAGES::Shop::API::Object::Shop');
    my $Customer = $Shop->path('Customers/TestCustomer');
    isa_ok($Customer, 'DE_EPAGES::Customer::API::Object::Customer' );

    my $CustomerOrder = $Shop->path('Customers/TestCustomer/Orders/1234-5678');
    _testOrder($CustomerOrder, 47, 88.70, 14.16, 'gross');
    $CustomerOrder = $Shop->path('Customers/TestCustomer/Orders/1234-5679');
    _testOrder($CustomerOrder, 47, 105.83, 16.90, 'net');
}

sub _testOrder {
    my ($CustomerOrder, $numberOfKeysExpected, $exp_amount, $exp_tax, $ExpectedTaxModel) = @_;
    isa_ok($CustomerOrder, 'DE_EPAGES::Order::API::Object::CustomerOrder' );

    my $Order = Bubo::makeOrder($CustomerOrder->container);

    my %nvp = $Order->as_nvp;
    is scalar keys %nvp, $numberOfKeysExpected, 'correct number of keys';
    check_nvp $ExpectedTaxModel, $Order, %nvp;
    #diag explain \%nvp;
    ok $Order->amount== $exp_amount, "total amount $exp_amount";
    ok $Order->tax == $exp_tax, "taxes $exp_tax";
    ok $nvp{order_amount} == $exp_amount, "total amount $exp_amount via NVP";
    ok $nvp{order_taxAmount} == $exp_tax, "taxes $exp_tax via NVP";
    is $nvp{order_currencyCode}, 'EUR', 'EUR';

}

sub Main {
    run_tests;
    done_testing;
}

RunScript( Sub => \&Main );

1;
